## -*- mode: autoconf -*-

##
## This file is part of the Cube software (http://www.scalasca.org)
##
## Copyright (c) 2015, 2016
## Forschungszentrum Juelich GmbH, Germany
##
## This software may be modified and distributed under the terms of
## a BSD-style license.  See the COPYING file in the package base
## directory for details.
##


##
## Internally used macros
##

dnl This macro compares the required interface version with the
dnl triplet common to libtool versioning 'current:revision:age'.
dnl
dnl Input:
dnl   $1 result variable to be set to "yes", "no", or "skipped"
dnl   $2 required interface version
dnl   $3 library interface version in format: current:revision:age
dnl
dnl Output:
dnl   The variable specified in $1 is modified.
dnl
AC_DEFUN([_AX_CHECK_SCOREP_SCORE_INTERFACE_VERSION],[
    m4_ifblank([$1], [m4_fatal([Missing result variable argument.])])
    m4_ifblank([$2], [m4_fatal([Missing required interface version argument.])])
    m4_ifblank([$3], [m4_fatal([Missing detected interface version argument.])])

    AC_MSG_CHECKING([interface version required])
    AC_MSG_RESULT([$2])
    
    scorep_max_provided_interface_version=`echo $3 | cut -d : -f 1`
    scorep_provided_age=`echo $3 | cut -d : -f 3`
    AC_MSG_CHECKING([interface version provided])
    AC_MSG_RESULT([$3])

    AC_MSG_CHECKING([whether provided interface is suitable])
    AS_IF([test "x${scorep_max_provided_interface_version}" != "x" && test "x${scorep_provided_age}" != "x"],
          [dnl perform the check
           AS_IF([test ${scorep_max_provided_interface_version} -eq 0 && test ${scorep_provided_age} -eq 0],
                 [dnl 0:0:0 indicates a development library. Skip check.
                  $1="skipped"
                 ],
                 [dnl do the math
                  AS_VAR_ARITH([scorep_min_provided_interface_version], [${scorep_max_provided_interface_version} - ${scorep_provided_age}])
                  AS_IF([test $2 -le ${scorep_max_provided_interface_version} && test $2 -ge ${scorep_min_provided_interface_version}],
                        [$1="yes"],
                        [$1="no"])
                 ])
          ],
          [$1="no"])

    AC_MSG_RESULT(${$1})
    AS_IF([test "x${$1}" = "xskipped"],              
          AC_MSG_WARN([The SCOREP_SCORE library found is an unversioned development library. You may experience compile-time and/or link-time errors.]))
])



dnl This macro performs queries to the config tool until variable is not-empty
dnl
dnl Input:
dnl   $1 prefix (lowercase)
dnl   $2 suffix (any case)
dnl   $3 comma separated list of query arguments to test
dnl
dnl Output:
dnl   A variable with all-uppercase name constructed from 'prefix_suffix' is
dnl   set to the result of the respective config-tool call. The variable is
dnl   also passed to AC_SUBST().
dnl
AC_DEFUN([_AX_SCOREP_SCORE_CONFIG_QUERY],[

    m4_ifblank([$1], [m4_fatal([Missing prefix argument.])])
    m4_ifblank([$2], [m4_fatal([Missing suffix argument.])])
    m4_ifblank([$3], [m4_fatal([Missing list of queries argument.])])
    
    AS_IF([test "x$4" = "x"],[ target_var=],[ target_var="--target $4"])
        
AC_MSG_CHECKING([value for m4_toupper($1_$2)])
    m4_foreach([_SCOREP_SCORE_QUERY_VAR], [$3], [
        AS_IF([test "x${m4_toupper($1_$2)}" = "x"],
              [dnl variable is unset; try next query
               scorep_config_query="${scorep_$1_config} --_SCOREP_SCORE_QUERY_VAR"
               scorep_config_query_output=`$scorep_config_query ${target_var} 2>/dev/null`
               AS_IF([test "x$scorep_config_query_output" != "x"],
                     [dnl query was successful; set variable
                      m4_toupper($1_$2)="$scorep_config_query_output"
                      AC_SUBST(m4_toupper($1_$2))
                      dnl AC_MSG_RESULT([${m4_toupper($1_$2)} (via --_SCOREP_SCORE_QUERY_VAR)])
                      AC_MSG_RESULT([${m4_toupper($1_$2)}])
                     ])
              ])
    ])
    AS_IF([test "x${m4_toupper($1_$2)}" = "x"],
          AC_MSG_RESULT([]))
])

dnl This macro checks for the availability of a config tool for the required component and
dnl validates the corresponding library interface version matches the requested interface
dnl version.
dnl
dnl Input:
dnl   $1 component short name (used as prefix to the config-tool and all corresponding variables)
dnl   $2 required library interface version
dnl   $3 configure option name (a more descriptive name used for the --with-<component> option)
dnl
dnl Output:
dnl   scorep_$1_config    : the detected config tool
dnl   scorep_$1_config_ok : either "yes", "no", or "skipped" depending on library interface
dnl
dnl The macro will check for the availablility of either '$1-config' or
dnl the deprecated 'cube-config'.
dnl
dnl The revision returned by the config tool must follow libtools naming 
dnl scheme 'current:revision:age'. The component found is accepted, iff
dnl current >= $2 && $2 >= (current - age).
dnl 
dnl If $with_val=yes, the config tool is searched in the PATH, if
dnl $with_val=no, the component is disabled, otherwise the config tool
dnl is searched in the provided path. 
dnl
AC_DEFUN([_AX_HAVE_SCOREP_SCORE_CONFIG_TOOL],[

    m4_ifblank([$1], [m4_fatal([The argument 1 may not be empty.])])
    m4_ifblank([$2], [m4_fatal([The argument 2 may not be empty.])])
    m4_ifblank([$3], [m4_fatal([The argument 3 may not be empty.])])

    AS_UNSET([scorep_$1_config])
    AS_UNSET([scorep_$1_config_ok])

    AC_ARG_WITH([$3],
        [AS_HELP_STRING([--with-$3[[=DIR]]],
            [Provide path to a pre-installed $1-config. Autodetected if already in $PATH.])],
        [AS_IF([test "x$withval" != "xno"],
              [AC_PATH_PROGS([scorep_$1_config],[$1-config],[],[$withval $withval/bin $PATH])])],
        [AC_PATH_PROGS([scorep_$1_config],[$1-config],[])])
        
    AS_IF([test "x${scorep_$1_config}" != "x"],
          [scorep_$1_config_ok="yes"],
          [scorep_$1_config_ok="no"])
          
    AS_IF([test "x${scorep_$1_config_ok}" != "no"],
          [dnl The library was found
           AC_DEFINE(AS_TR_CPP(m4_toupper([HAVE_$3])),[1],[Define if $3 support is available.])])
])

dnl This macro will search for a suitable config tool for the Cube library
dnl
dnl Input:
dnl   $1 required interface version
dnl
dnl Output:
dnl   Defines HAVE_SCOREP_SCORE
dnl
AC_DEFUN([AX_WITH_SCOREP_SCORE],[
    m4_ifblank([$1], [m4_fatal([$0: missing required library interface version])])

    _AX_HAVE_SCOREP_SCORE_CONFIG_TOOL([scorep], $1, [score-library])

    AS_IF([test "x${scorep_scorep_config_ok}" != "xno"],
          [dnl query variables
           _AX_SCOREP_SCORE_CONFIG_QUERY([scorep], [score_cppflags],       [cppflags,  cxxflags], [score])
           _AX_SCOREP_SCORE_CONFIG_QUERY([scorep], [score_include],        [include], [score])
           _AX_SCOREP_SCORE_CONFIG_QUERY([scorep], [score_ldflags],        [ldflags], [score])
           _AX_SCOREP_SCORE_CONFIG_QUERY([scorep], [score_libs],           [libs], [score])
          ], 
          [
          	AC_MSG_ERROR("Score-P's Score tool is required. One can specify path to scorep-config tool using command line option --with-scorep-library", [-1])
          ])
])
