/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "config.h"
#include "ScorePionToolBar.h"
#include "PluginServices.h"
#include <QToolButton>
#include <QMessageBox>
#include <QInputDialog>
#include <QMenu>
#include <assert.h>
#include "ScorePionPluginWidget.h"
#include "scorep/SCOREP_Score_Profile.hpp"
#include "scorep/SCOREP_Score_Estimator.hpp"


using namespace scorepion_plugin::gui;

ScorePionToolBar::ScorePionToolBar( cubepluginapi::PluginServices* service ) : QToolBar( "ScorePion Toolbar" )
{
    parent          = 0;
    this->service   = service;
    
    QAction * open_filter_action = this->addAction(QIcon(":/scorepionicons/open_filter.png"),"Open Filter file");
    connect(open_filter_action, SIGNAL(triggered()),
	    this, SLOT(openFilterFile()));
    QAction * save_filter_action = this->addAction(QIcon(":/scorepionicons/save_filter.png"),"Save Filter file");
    connect(save_filter_action, SIGNAL(triggered()),
	    this, SLOT(saveFilterFile()));
     
    addSeparator();
    
    add_incl_filter_action = this->addAction(QIcon(":/scorepionicons/add_to_filter.png"),"Add filter group \"include\"");
    QMenu * incl_filter_menu = new QMenu();
    QAction * incl_file = incl_filter_menu->addAction("Include File");
    connect(incl_file, SIGNAL(triggered()),
	    this, SLOT(addIncludeFile()));
    
    QAction * incl_region = incl_filter_menu->addAction("Include Region");
    connect(incl_region, SIGNAL(triggered()),
	    this, SLOT(addIncludeRegion()));
    add_incl_filter_action->setMenu( incl_filter_menu);
    
    add_excl_filter_action = this->addAction(QIcon(":/scorepionicons/remove_from_filter.png"),"Add filter group \"exclude\"");
    QMenu * excl_filter_menu = new QMenu();
    QAction * excl_file = excl_filter_menu->addAction("Exclude File");
    connect(excl_file, SIGNAL(triggered()),
	    this, SLOT(addExcludeFile()));
    
    QAction * excl_region = excl_filter_menu->addAction("Exclude Region");
    connect(excl_region, SIGNAL(triggered()),
	    this, SLOT(addExcludeRegion()));
    add_excl_filter_action->setMenu( excl_filter_menu);

    addSeparator();
    this->addWidget( new QLabel(tr("Number of counters ")) );
    num_counters_widget = new QSpinBox();
    num_counters_widget->setWhatsThis(tr("Number of additional performance counters to measure."));
    num_counters_widget->setToolTip(tr("Number of additional performance counters to measure."));
    
    this->addWidget( num_counters_widget );
    connect ( num_counters_widget, SIGNAL(valueChanged( int ) ), this, SLOT(changedCountersNumber( int )) );

    addSeparator();
    
    QAction * add_trace_estimation_metric_action = this->addAction(QIcon(":/scorepionicons/trace_estimation.png"),"Add OTF2 Trace size estimation metrics");
    connect(add_trace_estimation_metric_action, SIGNAL(triggered()),
	    this, SLOT(addMetric()));
      
    

}


void
ScorePionToolBar::setScorePionPluginWidget( ScorePionPluginWidget* newparent )
{
    parent = newparent;
}


// slots connected to the actions in the topology tool bar
void
ScorePionToolBar::showSelf()
{
    if ( parent != NULL )
    {
        this->show();
    }
}

void
ScorePionToolBar::hideSelf()
{
    this->hide();
}


void
ScorePionToolBar::addIncludeFile()
{
    /** Returns selected items in the given tree  @param type METRICTREE, CALLTREE, CALLFLAT, SYSTEMTREE */
    const QList<cubepluginapi::TreeItem*>& selected = service->getSelections( cubepluginapi::CALLTREE );
    
    std::vector<cube::Vertex * > v;
    foreach ( cubepluginapi::TreeItem* item, selected)
    {
        v.push_back( item->getCubeObject());
    }
    parent->addIncludeFile( v );
}


void
ScorePionToolBar::addIncludeRegion()
{
    /** Returns selected items in the given tree  @param type METRICTREE, CALLTREE, CALLFLAT, SYSTEMTREE */
    const QList<cubepluginapi::TreeItem*>& selected = service->getSelections( cubepluginapi::CALLTREE );
    
    std::vector<cube::Vertex * > v;
    foreach ( cubepluginapi::TreeItem* item, selected)
    {
        v.push_back( item->getCubeObject());
    }
    parent->addIncludeRegion( v );
}

void
ScorePionToolBar::addExcludeFile()
{
    /** Returns selected items in the given tree  @param type METRICTREE, CALLTREE, CALLFLAT, SYSTEMTREE */
    const QList<cubepluginapi::TreeItem*>& selected = service->getSelections( cubepluginapi::CALLTREE );
    
    std::vector<cube::Vertex * > v;
    foreach ( cubepluginapi::TreeItem* item, selected)
    {
        v.push_back( item->getCubeObject());
    }
    parent->addExcludeFile( v );
}

void
ScorePionToolBar::addExcludeRegion()
{
    /** Returns selected items in the given tree  @param type METRICTREE, CALLTREE, CALLFLAT, SYSTEMTREE */
    const QList<cubepluginapi::TreeItem*>& selected = service->getSelections( service->getActiveTreeType( cubepluginapi::CALL ) );
    
    std::vector<cube::Vertex * > v;
    foreach ( cubepluginapi::TreeItem* item, selected)
    {
        v.push_back( item->getCubeObject());
    }
    parent->addExcludeRegion( v );
}


void
ScorePionToolBar::saveFilterFile()
{
    parent->writeFilterFile( );
}


void
ScorePionToolBar::openFilterFile()
{
     parent->loadFilterFile();
}

void
ScorePionToolBar::addMetric()
{
    bool new_metric = false;
    cube::Metric* _total_met  = service->getCube()->get_met("total_full_trace_size");
    cube::Metric* _reduced_met = service->getCube()->get_met("total_reduced_trace_size");
    cube::Metric* _total_overhead_met  = service->getCube()->get_met("total_overhead");
    cube::Metric* _reduced_overhead_met = service->getCube()->get_met("reduced_overhead");
    if ( _total_met != NULL )
    {
        service->debug() << "Metric 'Total size of full trace' exists. Ignore. " << endl;
    }
    else
    {
        new_metric |= true;
         std::string cubepl_init_region_bytes = buildRegionTraceContributionMetric();
        _total_met = service->getCube()->def_met(
            "Total size of full trace",
            "total_full_trace_size",
            "INTEGER",
            "bytes",
            "",
            "",
            "",
            NULL,
            cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
            "${region_bytes}[${calculation::region::id}]*metric::visits()",
            cubepl_init_region_bytes
        );
      _total_met->setCacheable( false );
      service->addMetric( _total_met, NULL, true);      
    }
    
    if ( _reduced_met != NULL )
    {
        service->debug() << "Metric 'Total size of reduced trace' exists. Ignore. " << endl;
    }
    else
    { 
        new_metric |= true;
        _reduced_met = service->getCube()->def_met(
        "Total size of reduced trace",
        "total_reduced_trace_size",
        "INTEGER",
        "bytes",
        "",
        "",
        "",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        " metric::total_full_trace_size() * ${region_to_filter}[${calculation::region::id} ]"
        );    
        _reduced_met->setCacheable( false );
        service->addMetric( _reduced_met, NULL, true);
    }
    
    if ( _total_overhead_met != NULL )
    {
        service->debug() << "Metric 'Total overhead time' exists. Ignore. " << endl;
    }
    else
    {
        new_metric |= true;
         std::string cubepl_init_region_overhead = buildRegionOverheadMetric();
        _total_overhead_met = service->getCube()->def_met(
            "Total measurement overhead",
            "total_overhead",
            "DOUBLE",
            "sec",
            "",
            "",
            "",
            NULL,
            cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
            "${region_overhead}[${calculation::region::id}]*metric::visits()",
            cubepl_init_region_overhead
        );
      _total_overhead_met->setCacheable( false );
      service->addMetric( _total_overhead_met, NULL, true);      
    }
    
    if ( _reduced_overhead_met != NULL )
    {
        service->debug() << "Metric 'Reduced overhead time' exists. Ignore. " << endl;
    }
    else
    { 
        new_metric |= true;
        _reduced_overhead_met = service->getCube()->def_met(
        "Reduced measurement overhead",
        "reduced_overhead",
        "DOUBLE",
        "sec",
        "",
        "",
        "",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        " metric::total_overhead() * ${region_to_filter}[${calculation::region::id} ]"
        );    
        _reduced_overhead_met->setCacheable( false );
        service->addMetric( _reduced_overhead_met, NULL, true);
    }
       
    
    
    if (new_metric)
        emit addedMetric();
}

std::string
ScorePionToolBar::buildRegionTraceContributionMetric()
{
    // ----------------- build up list if contributions
    cube::Cube*    cube = service->getCube();

    SCOREP_Score_Profile* profile = new SCOREP_Score_Profile( cube );

    SCOREP_Score_Estimator * estimator = new SCOREP_Score_Estimator( profile, num_counters_widget->value() );

    const std::vector<cube::Region*> & regs = cube->get_regv();
    for ( std::vector<cube::Region*>::const_iterator iter = regs.begin(); iter != regs.end(); ++iter)
    {
        cube::Region* r = *iter;
        bytes_per_visit[ r] = estimator->bytesPerVisit( r->get_name());
    }
    delete profile;
    delete estimator;
    
    
    std::string cubepl_init_region_bytes= "{\n"
    "global(region_bytes);\n"
    "global(region_to_filter);\n";
    for ( std::map<cube::Region*, uint64_t>::iterator iter = bytes_per_visit.begin(); iter != bytes_per_visit.end(); ++iter)
    {
        std::ostringstream id_out;
        std::ostringstream b_out;
        std::string s_id;
        std::string s_bytes;
        cube::Region* r = iter->first;
        uint64_t id = r->get_id();
        uint64_t b = iter->second;
        id_out << id;
        s_id = id_out.str();
        b_out << b;
        s_bytes = b_out.str();
        cubepl_init_region_bytes += "${region_bytes}[" + s_id + "]=" + s_bytes + "; // " + r->get_name() + "\n";
        cubepl_init_region_bytes += "${region_to_filter}[" + s_id + "]=1;\n";
    }
    
    cubepl_init_region_bytes +="\n"
    "return 0;\n"
    "}\n";
    return cubepl_init_region_bytes;
     
}

std::string
ScorePionToolBar::buildRegionOverheadMetric()
{
    // ----------------- build up list if contributions
    cube::Cube*    cube = service->getCube();


    const std::vector<cube::Region*> & regs = cube->get_regv();
    for ( std::vector<cube::Region*>::const_iterator iter = regs.begin(); iter != regs.end(); ++iter)
    {
        cube::Region* r = *iter;
        overhead_per_visit[ r] = 0.000001;
    }
    
    
    std::string cubepl_init_region_overhead= "{\n"
    "global(region_overhead);\n";
    for ( std::map<cube::Region*, double>::iterator iter = overhead_per_visit.begin(); iter != overhead_per_visit.end(); ++iter)
    {
        std::ostringstream id_out;
        std::ostringstream b_out;
        std::string s_id;
        std::string s_bytes;
        cube::Region* r = iter->first;
        uint64_t id = r->get_id();
        double b = iter->second;
        id_out << id;
        s_id = id_out.str();
        b_out << std::fixed;
        b_out << b;
        s_bytes = b_out.str();
        cubepl_init_region_overhead += "${region_overhead}[" + s_id + "]=" + s_bytes + "; // " + r->get_name() + "\n";
    }
    
    cubepl_init_region_overhead +="\n"
    "return 0;\n"
    "}\n";
    return cubepl_init_region_overhead;
     
}



void
ScorePionToolBar::changedCountersNumber( int i )
{
    emit changedCountersNum( i );
}

